package AccordionBus;

import java.awt.*;
import java.awt.event.*;
import java.util.Random;
import javax.swing.*;

// Класс, отвечающий за прорисовку и перемещение объекта-сущности
public class DrawingBus {
    // Класс-сущность
    public EntityBus EntityBus;

    // Прорисовка дверей
    public IDrawingDoors drawingDoors;

    // Ширина окна
    private int _pictureWidth;

    // Высота окна
    private int _pictureHeight;

    // Левая координата прорисовки автобуса
    protected int _startPosX;

    // Координата X объекта
    public int GetPosX() { return _startPosX; }

    // Верхняя координата прорисовки автобуса
    protected int _startPosY;

    // Координата Y объекта
    public int GetPosY() { return _startPosY; }

    // Ширина прорисовки автобуса
    private int _busWidth = 100;

    // Ширина объекта
    public int GetWidth() { return _busWidth; }

    // Высота прорисовки автобуса
    private int _busHeight = 50;

    // Высота объекта
    public int GetHeigth() { return _busHeight; }

    // Проверка, что объект может перемещаться по указанному направлению
    public boolean CanMove(DirectionType direction) {
        if (EntityBus == null) {
            return false;
        }

        switch (direction) {
            // Влево
            case Left:
                return _startPosX - EntityBus.Step() > 0;
            // Вверх
            case Up:
                return  _startPosY - EntityBus.Step() > 0;
            // Вправо
            case Right:
                return _startPosX + EntityBus.Step() < _pictureWidth - _busWidth;
            // Вниз
            case Down:
                return _startPosY + EntityBus.Step() < _pictureHeight - _busHeight;
            default:
                return false;
        }
    }

    // Конструктор
    public DrawingBus(int speed, int weight, Color bodyColor, int width, int height) {
        // Проверка на вместимость объекта в размеры картинки
        if ((_busWidth >= width) || (_busHeight >= height)) {
            return;
        }

        _pictureWidth = width;
        _pictureHeight = height;
        EntityBus = new EntityBus(speed, weight, bodyColor);

        Random random = new Random();
        switch (random.nextInt(0, 3)) {
            case 0:
                drawingDoors = new DrawingDoorsTypeOne();
                break;
            case 1:
                drawingDoors = new DrawingDoorsTypeTwo();
                break;
            case 2:
                drawingDoors = new DrawingDoorsTypeThree();
                break;
            default:
                drawingDoors = new DrawingDoorsTypeOne();
                break;
        }
        drawingDoors.setNumDoors(random.nextInt(3, 6));
    }

    // Конструктор
    protected DrawingBus(int speed, double weight, Color bodyColor, int width, int height, int busWidth, int busHeight) {
        // Проверка на вместимость объекта в размеры картинки
        if ((_busWidth >= width) || (_busHeight >= height)) {
            return;
        }

        _pictureWidth = width;
        _pictureHeight = height;
        _busWidth = busWidth;
        _busHeight = busHeight;
        EntityBus = new EntityBus(speed, weight, bodyColor);

        Random random = new Random();
        switch (random.nextInt(0, 3)) {
            case 0:
                drawingDoors = new DrawingDoorsTypeOne();
                break;
            case 1:
                drawingDoors = new DrawingDoorsTypeTwo();
                break;
            case 2:
                drawingDoors = new DrawingDoorsTypeThree();
                break;
            default:
                drawingDoors = new DrawingDoorsTypeOne();
                break;
        }
        drawingDoors.setNumDoors(random.nextInt(3, 6));
    }

    // Установка позиции
    public void SetPosition(int x, int y) {
        if (x < 0 || y < 0) {
            x = 0;
            y = 0;
        }
        else if ((x + _busWidth > _pictureWidth) || (y + _busHeight > _pictureHeight)) {
            _startPosX = _pictureWidth - _busWidth;
            _startPosY = _pictureHeight - _busHeight;
        }
        else {
            _startPosX = x;
            _startPosY = y;
        }
    }

    // Изменение направления перемещения
    public void MoveTransport(DirectionType direction) {
        if (!CanMove(direction) || EntityBus == null) {
            return;
        }

        switch (direction) {
            // Влево
            case Left:
                _startPosX -= (int)EntityBus.Step();
                break;
            // Вверх
            case Up:
                _startPosY -= (int)EntityBus.Step();
                break;
            // Вправо
            case Right:
                _startPosX += (int)EntityBus.Step();
                break;
            // Вниз
            case Down:
                _startPosY += (int)EntityBus.Step();
                break;
        }
    }

    // Прорисовка объекта
    public void DrawTransport(Graphics g) {
        if (EntityBus == null)
        {
            return;
        }

        Graphics2D g2d = (Graphics2D)g;

        Color bodyColor = EntityBus.getBodyColor();
        Color blackColor = Color.BLACK;
        Color whiteColor = Color.WHITE;

        // Граница главного отсека
        g2d.setColor(blackColor);
        g2d.setStroke(new BasicStroke(3));
        g2d.drawRect(_startPosX, _startPosY, 100, 40);

        // Кузов главного отсека
        g2d.setColor(bodyColor);
        g2d.fillRect(_startPosX + 1, _startPosY + 1, 99, 39);

        // Окна главного отсека
        g2d.setColor(Color.CYAN);
        g2d.fillRect(_startPosX, _startPosY + 5, 100, 15);
        g2d.setColor(blackColor);
        g2d.setStroke(new BasicStroke(2));
        g2d.drawLine(_startPosX, _startPosY + 5, _startPosX + 100, _startPosY + 5);
        g2d.drawLine(_startPosX, _startPosY + 20, _startPosX + 100, _startPosY + 20);

        // Дверь главного отсека
        g2d.setColor(blackColor);
        g2d.setStroke(new BasicStroke(2));
        g2d.drawRect(_startPosX + 10, _startPosY + 10, 15, 30);
        g2d.setColor(whiteColor);
        g2d.fillRect(_startPosX + 11, _startPosY + 11, 14, 29);

        // Дополнительные двери
        drawingDoors.DrawDoors(g2d, bodyColor, false, _startPosX, _startPosY);

        // Колеса главного отсека
        g2d.setColor(blackColor);
        g2d.setStroke(new BasicStroke(3));
        g2d.drawOval(_startPosX + 15, _startPosY + 33, 15, 15);
        g2d.drawOval(_startPosX + 70, _startPosY + 33, 15, 15);
        g2d.setColor(whiteColor);
        g2d.fillOval(_startPosX + 15, _startPosY + 33, 15, 15);
        g2d.fillOval(_startPosX + 70, _startPosY + 33, 15, 15);
    }
}
